#!/bin/bash

# This script was used for generating a patch for screen-3.9.9-alt3 
# package. For more info on what it does, look at the text at the end of it.
#
# July 2001, imz@altlinux.ru

# what to look for and replace; the pattern should be Ok for egrep and perl
pattern='\.(screen([^[:alnum:]_-]|$))'

# a regex for find that specifies the files we wnat to ignore when 
# doing the search and replace
filesToIgnore='.*\(\.orig$\|/Changelog$\)'

function usage() {
cat >&2 <<EOF
"$0" (--check|--show|--gen) srcdir

This script generates a patch for screen sources.
The patch replaces occurences (not all) of ".screen"
with "tmp/screen". For more info, look inside the script.

Arguments:
srcdir: the directory where the sources (of screen) were unpacked

Actions:

--check: search & report where there are any occurences of ".screen" that would 
be replaced;

--show: search & show these occurences;

--gen: replace & generate the patch (diff -duNr); no copy of the 
	original source is kept.
EOF
}

function check() {
if egrep --quiet "$pattern" "${files[@]}"; then
    echo "While searching $srcdir, \".screen\" matches were found; probably you should generate \
a new patch by running 
	\"$0\" --gen \"$srcdir\"
from $(pwd).
"
    exit 1
else
    exit 0
fi
}

function show() {
egrep --with-filename "$pattern" "${files[@]}"
}

function gen() {
unset LANG LANGUAGE
export LANG LANGUAGE

local origcopy="$(mktemp -d "$srcdir.orig-XXXXXX")/$(basename "$srcdir")"

[ -e "$origcopy" ] && {
    echo "$origcopy already exists (we need it)" >&2
    exit 1
}

cp -a "$srcdir" "$origcopy"

perl -pi -e "s!$pattern!tmp/\\1!" "${files[@]}"

fold --space <<EOF
This diff was automatically generated for "$srcdir", \
with occurences of ".screen" replaced with "tmp/screen". \
(This is the place inside a user's homedir where the \
user's screen sockets are located.) The generating \
script tries to act intelligently (not to change words \
that include ".screen" as a substring); so not all \
the occurences of ".screen" are replaced, but still it is \
possible that more occurences of it have been replaced than \
you would like to.

Thus, before you use this diff as a patch, LOOK THROUGH it \
and EXAMINE THE CHANGES!

$(date), $LOGNAME@$(hostname).
EOF

diff -duNr "$origcopy" "$srcdir"

rm -rf "$origcopy"
# Remove the dir created by mktemp -d:
rm -rf "$(dirname "$origcopy")"
}

case "$1" in
--check|--show|--gen)
    if (( $# != 2 )); then
	echo "We expect one argument. You gave $(( $# - 1 ))!" >&2
	exit 1
    elif [[ ( "$2" == "." ) || ( "$2" == ".." ) ]]; then
	echo "We do not work with either . or ..!" >&2
	exit 1
    else
	srcdir="$2"
    fi
    # The lisrt of files to process:
    # regular files in . (not symlinks),
    # and with no trailing ".orig" suffix
    IFS=$'\n' files=($(find "$srcdir" -type f ! -regex "$filesToIgnore"))
    ${1#--}
    ;;
*)
    usage
    exit 1
    ;;
esac
